/* This file is part of the KDE project
   Copyright (C) 2004 Jaroslaw Staniek <js@iidea.pl>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "kexiviewbase.h"

#include "keximainwindow.h"
#include "kexidialogbase.h"
#include "kexipropertybuffer.h"
#include "kexiproject.h"
#include "kexi_utils.h"

#include <kexidb/connection.h>
#include <kexidb/utils.h>
#include <kdebug.h>

KexiViewBase::KexiViewBase(KexiMainWindow *mainWin, QWidget *parent, const char *name)
 : QWidget(parent, name)
 , KexiActionProxy(this, mainWin)
 , m_mainWin(mainWin)
 , m_dirty(false)
 , m_viewWidget(0)
 , m_parentView(0)
{
	QWidget *wi=this;
	while ((wi = wi->parentWidget()) && !wi->inherits("KexiDialogBase"))
		;
	m_dialog = (wi && wi->inherits("KexiDialogBase")) ? static_cast<KexiDialogBase*>(wi) : 0;

	installEventFilter(this);
}

KexiViewBase::~KexiViewBase()
{
}

bool KexiViewBase::beforeSwitchTo(int /* mode */, bool & /*cancelled*/, bool & /*dontStore*/)
{
	return true;
}

bool KexiViewBase::afterSwitchFrom(int /* mode */, bool & /*cancelled*/)
{
	return true;
}

QSize KexiViewBase::preferredSizeHint(const QSize& otherSize)
{
	KexiDialogBase* dlg = parentDialog();
	if (dlg && dlg->mdiParent()) {
		QRect r = dlg->mdiParent()->mdiAreaContentsRect();
		return otherSize.boundedTo( QSize(
			r.width() - 10,
			r.height() - dlg->mdiParent()->captionHeight() - dlg->pos().y() - 10
		) );
	}
	return otherSize;
}

void KexiViewBase::closeEvent( QCloseEvent * e )
{
	emit closing();
	QWidget::closeEvent(e);
}

KexiPropertyBuffer *KexiViewBase::propertyBuffer()
{
	return 0;
}

void KexiViewBase::propertyBufferSwitched()
{
	if (parentDialog())
		m_mainWin->propertyBufferSwitched( parentDialog(), false );
}

void KexiViewBase::propertyBufferReloaded()
{
	if (parentDialog())
		m_mainWin->propertyBufferSwitched( parentDialog(), true );
}

void KexiViewBase::setDirty(bool set)
{
	if (m_dirty == set)
		return;
	m_dirty = set;
	m_dirty = dirty();
	if (m_dirty!=set)//eventually didn't change
		return;
	if (m_dialog)
		m_dialog->dirtyChanged();
}

/*bool KexiViewBase::saveData()
{
	//TODO....

	//finally:
	setDirty(false);
	return true;
}*/

/*virtual*/
KexiDB::SchemaData* KexiViewBase::storeNewData(const KexiDB::SchemaData& sdata)
{
	KexiDB::SchemaData *new_schema = new KexiDB::SchemaData(sdata);

	if (!m_mainWin->project()->dbConnection()
			->storeObjectSchemaData( *new_schema, true /*newObject*/ ))
	{
		delete new_schema;
		new_schema=0;
	}
	else {
		setDirty(false);
	}
	return new_schema;
}

bool KexiViewBase::storeData()
{
	if (!m_dialog->schemaData())
		return false;
	if (!m_mainWin->project()->dbConnection()
			->storeObjectSchemaData( *m_dialog->schemaData(), false /*existing object*/ ))
	{
		return false;
	}
	setDirty(false);
	return true;
}

bool KexiViewBase::storeDataBlock( const QString &dataString, const QString &id, const QString& dataID )
{
	KexiDB::Connection *conn = m_mainWin->project()->dbConnection();
	KexiDB::Driver *drv = conn->driver();
	
	QString sql = "select kexi__objectdata.o_id from kexi__objectdata where o_id=" + id;
	QString sql_sub = KexiDB::sqlWhere(drv, KexiDB::Field::Text, "o_sub_id", dataID);

	bool ok, exists;
	exists = conn->resultExists(sql + " and " + sql_sub, ok);
	if (!ok)
		return false;
	if (exists) {
		return conn->drv_executeSQL( "update kexi__objectdata set o_data="
			+ drv->valueToSQL( KexiDB::Field::BLOB, dataString )
			+ " where o_id=" + id + " and " + sql_sub );
	}
	return conn->drv_executeSQL( 
		"insert into kexi__objectdata (o_id, o_data, o_sub_id) values ("
		+ id +"," + drv->valueToSQL( KexiDB::Field::BLOB, dataString )
		+ "," + drv->valueToSQL( KexiDB::Field::Text, dataID ) + ")" );
}

bool KexiViewBase::loadDataBlock( QString &dataString, const QString &id, const QString& dataID )
{
	KexiDB::Connection *conn = m_mainWin->project()->dbConnection();
	KexiDB::Driver *drv = conn->driver();

	return conn->querySingleString(
		QString("select o_data from kexi__objectdata where o_id=") + id
		+ " and " + KexiDB::sqlWhere(drv, KexiDB::Field::Text, "o_sub_id", dataID), dataString );
}


bool KexiViewBase::eventFilter( QObject *o, QEvent *e )
{
	if (e->type()==QEvent::FocusIn || e->type()==QEvent::FocusOut) {// && o->inherits("QWidget")) {
//		//hp==true if currently focused widget is a child of this table view
//		const bool hp = Kexi::hasParent( static_cast<QWidget*>(o), focusWidget());
		if (Kexi::hasParent( this, static_cast<QWidget*>(o))) {
			if (e->type()==QEvent::FocusOut && focusWidget() && !Kexi::hasParent( this, focusWidget())) {
				//focus out: when currently focused widget is not a parent of this view
				emit focus(false);
			} else if (e->type()==QEvent::FocusIn) {
				emit focus(true);
			}
			if (e->type()==QEvent::FocusOut) { // && focusWidget() && Kexi::hasParent( this, focusWidget())) { // && focusWidget()->inherits("KexiViewBase")) {
//				kdDebug() << focusWidget()->className() << " " << focusWidget()->name()<< endl;
				kdDebug() << o->className() << " " << o->name()<< endl;
				KexiViewBase *v = Kexi::findParent<KexiViewBase>(o, "KexiViewBase") ;
				if (v) {
					while (v->m_parentView)
						v = v->m_parentView;
					v->m_lastFocusedChildBeforeFocusOut = static_cast<QWidget*>(o); //focusWidget();
				}
			}

			if (e->type()==QEvent::FocusIn && m_actionProxyParent) {
				m_actionProxyParent->m_focusedChild = this;
			}
		}
	}
	return false;
}

void KexiViewBase::setViewWidget(QWidget* w)
{
	if (m_viewWidget == w)
		return;
	m_viewWidget = w;
	if (m_viewWidget) {
		m_viewWidget->installEventFilter(this);
	}
}

void KexiViewBase::addChildView( KexiViewBase* childView )
{
	addActionProxyChild( childView );
	childView->m_parentView = this;
//	if (m_parentView)
//		childView->installEventFilter(m_parentView);
	childView->installEventFilter(this);

}

void KexiViewBase::setFocus()
{
	if (!m_lastFocusedChildBeforeFocusOut.isNull()) {
		kdDebug() << "FOCUS: " << m_lastFocusedChildBeforeFocusOut->className() << " " << m_lastFocusedChildBeforeFocusOut->name()<< endl;
		m_lastFocusedChildBeforeFocusOut->setFocus();
	}
	else
		QWidget::setFocus();
}


#include "kexiviewbase.moc"

